@@ -658,6 +658,10 @@ get_gnome_keyring_password (GksuContext *context)
       password = g_locale_from_utf8 (found->secret,
 				     password_length,
 				     NULL, NULL, NULL);
+      password_length = strlen (password);
+
+      if (password[password_length-1] == '\n')
+	password[password_length-1] = '\0';
       return password;
     }
 
@@ -1334,6 +1338,9 @@ gksu_context_new ()
   context->debug = FALSE;
 
   context->sn_context = NULL;
+  context->sn_id = NULL;
+  
+  context->ref_count = 1;
 
   get_configuration_options (context);
   startup_notification_initialize (context);
@@ -1636,6 +1643,20 @@ gksu_context_get_launcher_context (GksuContext *context)
   return context->sn_context;
 }
 
+/**
+ * gksu_context_set_launcher_id:
+ * @context: the #GksuContext you want to set the sn id in
+ * @sn_context: the sn_id you want to set, as a #gchar
+ */
+void
+gksu_context_set_launcher_id (GksuContext *context,
+			      gchar *sn_id)
+{
+  if (context->sn_id)
+    g_free (context->sn_id);
+  context->sn_id = g_strdup(sn_id);
+}
+
 /**
  * gksu_context_launch_initiate:
  * @context: the #GksuContext you want to initiate the launch for
@@ -1646,13 +1667,23 @@ gksu_context_get_launcher_context (GksuContext *context)
 static void
 gksu_context_launch_initiate (GksuContext *context)
 {
-  gchar *sid = g_strdup_printf ("%s", sn_launcher_context_get_startup_id (context->sn_context));
+  gchar *sid = NULL;
   guint32 launch_time = gdk_x11_display_get_user_time ((GdkDisplay*)GDK_DISPLAY());
+  static gboolean initiated = FALSE;
+
+  if (!initiated)
+    initiated = TRUE;
+  else
+    return;
 
   sn_launcher_context_initiate (context->sn_context,
 				g_get_prgname (),
 				gksu_context_get_command (context),
 				launch_time);
+
+  sid = g_strdup_printf ("%s", sn_launcher_context_get_startup_id (context->sn_context));
+  gksu_context_set_launcher_id (context, sid);
+
   if (context->debug)
     fprintf (stderr, "STARTUP_ID: %s\n", sid);
   setenv ("DESKTOP_STARTUP_ID", sid, TRUE);
@@ -1724,6 +1755,52 @@ gksu_context_free (GksuContext *context)
   g_free (context);
 }
 
+/**
+ * gksu_context_ref
+ * @context: A #GksuContext struct.
+ *
+ * Increments the reference count of the given #GksuContext.
+ */
+GksuContext*
+gksu_context_ref (GksuContext *context)
+{
+  context->ref_count++;
+  return context;
+}
+
+/**
+ * gksu_context_unref
+ * @context: A #GksuContext struct.
+ *
+ * Decrements the reference count of the given #GksuContext struct,
+ * freeing it if the reference count falls to zero.
+ */
+void
+gksu_context_unref (GksuContext *context)
+{
+  if (--context->ref_count == 0)
+    {
+        gksu_context_free (context);
+    }
+}
+
+GType
+gksu_context_get_type (void)
+{
+  static GType type_gksu_context = 0;
+
+  if (!type_gksu_context)
+    type_gksu_context = g_boxed_type_register_static
+      ("GksuContext", 
+       (GBoxedCopyFunc) gksu_context_ref,
+       (GBoxedFreeFunc) gksu_context_unref);
+
+  return type_gksu_context;
+}
+
+
+
+
 /**
  * gksu_su_full:
  * @context: a #GksuContext
@@ -2000,6 +2077,10 @@ gksu_su_full (GksuContext *context,
 	      write (fdpty, line, strlen(line));
 	      g_free (line);
 
+	      line = g_strdup_printf ("gksu-run: %s\n", context->sn_id);
+	      write (fdpty, line, strlen(line));
+	      g_free (line);
+
 	      line = g_strdup_printf ("gksu-run: %s\n", context->xauth);
 	      write (fdpty, line, strlen(line));
 	      g_free (line);
